@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:addr.c,v 1.26 1998/07/05 17:51:47 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:addr.c,v 1.27 1998/09/21 05:45:48 woods Exp"
  */
 
 /*
@@ -492,13 +492,8 @@ parse_address(address, target, remainder, flagp)
      */
     if (target) {
 	*remainder = ap;
-#ifndef USE_TARGET_DOMAIN
-	*target = NULL;
-	DEBUG1(DBG_ADDR_HI, "parse_address: LOCAL: remainder=%s\n", *remainder);
-#else
 	DEBUG2(DBG_ADDR_HI, "parse_address: LOCAL: target=%s, remainder=%s\n",
 	       *target ? *target : "(no-domain)", *remainder);
-#endif
     } else {
 	DEBUG(DBG_ADDR_HI, "parse_address: LOCAL\n");
     }
