@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:bindlib.c,v 1.48 1998/06/08 01:56:07 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:bindlib.c,v 1.50 1998/10/01 05:25:26 woods Exp"
  */
 
 /*
@@ -349,6 +349,7 @@ struct mx_work_area {
     int * no_server;                    /* Is the server dead? */
 };
 
+static struct bindlib_private generic_bindattr = BIND_TEMPLATE_ATTRIBUTES;
 
 /*
  * import h_errno; many systems don't define it in <netdb.h>
@@ -391,6 +392,7 @@ static RR *getnextrr P_((struct rr_iterator*,int));
 static void rewindrr P_((struct rr_iterator*,HEADER*,int));
 static void do_header_rewrites P_((char *, char *));
 static int find_a_records P_((struct mx_transport_hint*,char*,char **));
+static struct error *not_a_canonical_host P_((char*,char*));
 static struct error *server_failure P_((char*,char*));
 static struct error *packet_error P_((char*,char*,char*));
 static struct error *matched_local_host P_((char*,char*));
@@ -440,13 +442,114 @@ bind_compute_domain ()
 
 
 /*
- * bind_check_if_local_mxs - check if localhost is MXing for target
+ * bind_check_if_canonical_host - check if domain is a canonical hostname and
+ *				  its address matches inet
  *
  * Return one of the following values:
  *
  * These return codes apply only to the specific address:
- *      DB_SUCCEED+1	Yes, we MX for the target host.
- *	DB_SUCCEED	No, we do not MX for the target host.
+ *      DB_SUCCEED	Yes, it is a canonical hostname.
+ *      DB_NOMATCH	No, it is not a canonical hostname
+ *      DB_FAIL		Negative response from server indicating some
+ *			problem with the request.
+ *      DB_AGAIN	Negative respone from server indicating a
+ *			temporary problem.  Try again later.
+ *
+ * These return codes apply to this router in general:
+ *      FILE_NOMATCH	Could not connect to the server.
+ *      FILE_AGAIN	Lost contact with server, or other failure
+ *			talking to server.  Try again later.
+ *      FILE_FAIL	A major error has been caught in router,
+ *			notify postmaster.
+ */
+
+#define CANONICAL_HOST_CHECK	"canonical host check"
+
+int
+bind_check_if_canonical_host(hostnm, inet, error_p)
+    char *hostnm;			/* domain name of host */
+    unsigned long inet;			/* expected internet address for host */
+    struct error **error_p;		/* pointer to error information */
+{
+    long save_res_options;		/* like _res.options in <resolv.h> */
+    int no_server = FALSE;		/* TRUE if no server process */
+    int found_server = FALSE;		/* TRUE if server process found */
+    char *error_msg = NULL;		/* pointer for error string from find_a_records() */
+    HEADER *a_rrs;
+    int a_size;
+    struct rr_iterator a_it;
+    RR *a_rr;
+    int result;
+    int nothing;
+
+    DEBUG1(DBG_DRIVER_MID, "bind_check_if_canonical_host(%s) called.\n", hostnm);
+
+#ifdef RES_INIT
+    /*  This initialises the resolver.  If the resolver is so old that RES_INIT
+     *  isn't defined then the routines will initialise themselves - that is
+     *  if the thing will compile at all!
+     */
+    if ((_res.options & RES_INIT) == 0) {
+	if (res_init() == EOF) {
+	    DEBUG(DBG_DRIVER_LO, "ERROR: res_init() failed - deferring addresses\n");
+	    *error_p = res_init_error(CANONICAL_HOST_CHECK);
+	    return FILE_AGAIN;
+	}
+    }
+#endif
+    save_res_options = _res.options;
+
+    _res.options &= ~(RES_DEFNAMES|RES_DNSRCH);
+    a_rrs = (HEADER *) xmalloc(MAXPACKET);
+
+    result = get_records(hostnm, T_A, a_rrs, &a_size, &error_msg);
+
+    if (result != DB_SUCCEED) {
+        xfree((char *) a_rrs);
+	*error_p = server_failure(CANONICAL_HOST_CHECK, error_msg);
+        return result;
+    }
+
+    result = DB_NOMATCH;		/* assume no data, or no A RRs, in response */
+    nothing = TRUE;			/* assume no data in response */
+    *error_p = NULL;
+    rewindrr(&a_it, a_rrs, a_size);
+    while ((a_rr = getnextrr(&a_it, FALSE)) != 0) {
+	nothing = FALSE;		/* must be some data here.... */
+	if (EQIC(a_it.dname, hostnm)) {
+	    /* XXX I've no idea why ((unsigned long) (a_rr->rr_data) == inet) doesn't work XXX */
+	    if ((a_rr->rr_type == T_A) && (bcmp(a_rr->rr_data, (char *) &inet, sizeof(inet)) == 0)) {
+		result = DB_SUCCEED;	/* Oh!  I guess there was a matching A RR! */
+	    } else if (a_rr->rr_type == T_CNAME) {
+		*error_p = not_a_canonical_host(hostnm, "domain name is a CNAME");
+	    } /* we don't care about any other kind of RRs */
+	}
+    }
+    if (result != DB_SUCCEED && !*error_p) {
+	*error_p = not_a_canonical_host(hostnm,
+					xprintf("no A records matching that hostname for target address of %s",
+						inet_ntoa(inet_makeaddr(inet, (unsigned long) 0))));
+    } else if (nothing) {
+	*error_p = not_a_canonical_host(hostnm, "no response to query for hostname");
+    }
+    xfree((char *) a_rrs);
+    _res.options = save_res_options;
+
+    return result;
+}
+
+
+
+/*
+ * bind_check_if_local_mxs - check if localhost is MXing for target
+ * 
+ * If DB_SUCCEED && (*local_precedence_p >= 0) then yes, we do MX for the
+ * target.
+ * 
+ * Return one of the following values:
+ *
+ * These return codes apply only to the specific address:
+ *	DB_SUCCEED	Lookup successful
  *      DB_NOMATCH	There is no such domain.
  *      DB_FAIL		Negative response from server indicating some
  *			problem with the request.
@@ -463,11 +566,10 @@ bind_compute_domain ()
 
 #define LOCALHOST_MX_CHECK	"localhost MX check"
 
-static struct bindlib_private localhost_bindattr = BIND_TEMPLATE_ATTRIBUTES;
-
 int
-bind_check_if_local_mxs(cur)
+bind_check_if_local_mxs(cur, local_precedence_p)
     struct addr *cur;
+    int *local_precedence_p;
 {
     long save_res_options;		/* like _res.options in <resolv.h> */
     char *strip_target;			/* target stripped of extra dots */
@@ -504,14 +606,14 @@ bind_check_if_local_mxs(cur)
 #  define RES_DNSRCH	0
 # endif
     if (!strchr(cur->target, '.')) {
-	_res.options |= RES_DEFNAMES|RES_DNSRCH;
+	_res.options |= RES_DEFNAMES|RES_DNSRCH; /* XXX this may not be a good idea here.... */
     } else {
 	_res.options &= ~(RES_DEFNAMES|RES_DNSRCH);
     }
 #endif
     strip_target = strip_dots(cur->target); /* Omit needless dots! */
 
-    mxdat = mx_work_area_constructor(&localhost_bindattr,
+    mxdat = mx_work_area_constructor(&generic_bindattr,
 				     &cur->error, LOCALHOST_MX_CHECK, 0L,
 				     &found_server, &no_server);
 
@@ -536,7 +638,8 @@ bind_check_if_local_mxs(cur)
 
         return ret;
     }
-    ret = (mxdat->local_precedence >= 0) ? (DB_SUCCEED + 1) : DB_SUCCEED;
+    ret = DB_SUCCEED;
+    *local_precedence_p = mxdat->local_precedence;
 
     mx_work_area_destructor(mxdat, FALSE);
     _res.options = save_res_options;
@@ -1782,17 +1885,20 @@ get_records(qname, qtype, answer, pack_size, error)
     int anslen;
 
     *pack_size = 0;                     /* Ensure packet size is zero unless set */
+    *error = NULL;
 #ifdef OBSOLETE_RESOLVER
     msglen = res_mkquery(QUERY, qname, C_IN, qtype, (char *)NULL, 0,
                          (struct rrec *)NULL, msgbuf, MAXPACKET);
 
     anslen = res_send(msgbuf, msglen,  (char *)answer, MAXPACKET);
     if (anslen < 0) {
+	*error = xprintf("DNS resolver error: %s", strerror(errno));
         return FILE_NOMATCH;
     }
 #else                                   /* not OBSOLETE_RESOLVER */
-    anslen = res_search(qname, C_IN, qtype, (u_char *)answer, MAXPACKET);
+    anslen = res_query(qname, C_IN, qtype, (u_char *) answer, MAXPACKET);
     if (anslen < 0) {
+	*error = (char *) hstrerror(h_errno); /* XXX cast is to explicitly discard const */
         switch (h_errno) {
         case NO_DATA:
 	    /*
@@ -1811,19 +1917,16 @@ get_records(qname, qtype, answer, pack_size, error)
             return DB_NOMATCH;
 
         case TRY_AGAIN:
-            *error = "Nameserver: Server failure";
             return DB_AGAIN;
 
         case NO_RECOVERY:
-            *error = "Irrecoverable nameserver error";
             return FILE_NOMATCH;
 
         default:
-            *error = "Unknown nameserver error";
             return FILE_NOMATCH;
         }
     }
-#endif                                  /* not OBSOLETE_RESOLVER */
+#endif /* not OBSOLETE_RESOLVER */
     *pack_size = anslen;
 
     answer->qdcount = ntohs(answer->qdcount);
@@ -1832,31 +1935,33 @@ get_records(qname, qtype, answer, pack_size, error)
     answer->arcount = ntohs(answer->arcount);
 
     switch (answer->rcode) {
-
-      case NOERROR:
+    case NOERROR:
         return DB_SUCCEED;
+
 #ifdef OBSOLETE_RESOLVER
-      case FORMERR:
-        *error = "Nameserver: Format error in packet";
+    case FORMERR:
+        *error = "DNS resolver: Format error in packet";
         return DB_FAIL;
 
-      case SERVFAIL:
-        *error = "Nameserver: Server failure";
+    case SERVFAIL:
+        *error = "DNS resolver: Server failure";
         return DB_AGAIN;
 
-      case NXDOMAIN:
+    case NXDOMAIN:
+	*error = "DNS resolver: no such domain name";
         return DB_NOMATCH;
 
-      case NOTIMP:
-        *error = "Nameserver: Unimplemented request";
+    case NOTIMP:
+        *error = "DNS resolver: Unimplemented request";
         return DB_FAIL;
 
-      case REFUSED:
-        *error = "Nameserver: Query refused";
+    case REFUSED:
+        *error = "DNS resolver: Query refused";
         return FILE_AGAIN;
-#endif                                  /* OBSOLETE_RESOLVER */
-      default:
-        *error = "Nameserver: Unknown response code";
+#endif /* OBSOLETE_RESOLVER */
+
+    default:
+        *error = "DNS resolver: Unknown response code";
         return DB_FAIL;
     }
 }
@@ -2014,6 +2119,33 @@ decode_mx_rr(rr, it, name, precedence)
     return DB_SUCCEED;
 }
 
+
+static struct error *
+not_a_canonical_host(host, error)
+    char *host;
+    char *error;                        /* additional error text */
+{
+    char *error_text;
+
+    extern int errno;
+
+    /*
+     * ERR_186 - canonical host not found
+     *
+     * DESCRIPTION
+     *      No A RR was returned for the domain queried.
+     *
+     * ACTIONS
+     *      Actions depend upon the specific error.
+     *
+     * RESOLUTION
+     *      Resolution depends upon the specific error.
+     */
+    error_text = xprintf("%s: %s", host, error);
+    DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
+
+    return note_error(ERR_NPOSTMAST | ERR_186, error_text);
+}
 
 static struct error *
 server_failure(what, error)
@@ -2025,7 +2157,7 @@ server_failure(what, error)
     extern int errno;
 
     /*
-     * ERR_164 - failure talking to BIND server
+     * ERR_164 - failure talking to DNS server
      *
      * DESCRIPTION
      *      An error occured when sending or receiving packets from
@@ -2039,8 +2171,11 @@ server_failure(what, error)
      * RESOLUTION
      *      Resolution depends upon the specific error.
      */
-    error_text = xprintf("%s: BIND server failure: %s: %s",
-                         what, error, strerror(errno));
+    error_text = xprintf("%s: DNS error: %s%s%s",
+			 what,
+			 error,
+			 (errno != 0) ? ": " : "",
+			 (errno != 0) ? strerror(errno) : "");
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_NPOSTMAST | ERR_164, error_text);
@@ -2066,7 +2201,9 @@ packet_error(what, type, host)
      *      away.
      *
      * RESOLUTION
-     *      This is probably a bug in the nameserver.
+     *      This is probably a bug in the nameserver, or some as yet
+     *      unhandled boundary condition in the way we parse the response
+     *      packet.  If the latter then it's a bug in smail.
      */
     error_text =
         xprintf("%s: BIND server format error in %s packet for %s",
@@ -2115,8 +2252,7 @@ no_transport_error(what, transport)
      * ERR_110 - no transport for router
      *
      * DESCRIPTION
-     *      A format error was found in a packet returned by the BIND
-     *      name server.
+     *      Router specified an illegal transport name
      *
      * ACTIONS
      *      Defer delivery.
@@ -2202,7 +2338,6 @@ res_init_error(what)
      *      An attempt to use res_init() to setup the BIND resolver
      *      failed.
      *      
-     *
      * ACTIONS
      *      There is something badly wrong with the resolver
      *
