@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:config.h,v 1.60 1998/04/11 23:30:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:config.h,v 1.61 1998/10/07 05:25:52 woods Exp"
  */
 
 /*
@@ -768,6 +768,14 @@ ${if and{{origin:local} {def:owner}}\
 # define SMTP_RECEIVE_MESSAGE_TIMEOUT	(2*60*60)	/* 2 hours */
 #endif
 
+/* 
+ * The default list of networks from which to allow remote relay and
+ * which do not need/want RCPT TO address to be verified.
+ */
+#ifndef SMTP_RECIPIENT_NO_VERIFY
+# define SMTP_RECIPIENT_NO_VERIFY	NULL
+#endif
+
 /*
  * The default list of networks from which to allow remote relay.
  *
