@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:direct.c,v 1.23 1998/04/11 23:30:20 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:direct.c,v 1.24 1998/09/21 05:44:18 woods Exp"
  */
 
 /*
@@ -633,10 +633,11 @@ finish_direct_other_form(dp, addr)
 
 	    return SEND_TO_NEXT;
 	}
-#if 0 /* since we no longer reparse mailbox forms of localhosts right down to the remainder */
-	/* This prevents accidental matching of remainder for virtual hosts */
+	/* Turning on ADDR_DONTHASH prevents accidental matching of remainder
+	 * for virtual hosts or for cases where an alias/etc. re-directs to a
+	 * localhost.
+	 */
 	addr->flags |= ADDR_DONTHASH;
-#endif
 	DEBUG6(DBG_DIRECT_HI,
 	       "    NOT passing %s@%s to next director as %s%s%s%s\n",
 	       addr->parent->remainder,
