@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_102:aliasfile.c,v 1.17 1997/07/01 01:32:23 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_103:aliasfile.c,v 1.19 1998/10/07 05:38:36 woods Exp"
  */
 
 /*
@@ -152,8 +152,9 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 	return in;
     }
 
-    if (!(priv = (struct aliasfile_private *)dp->private))
+    if (!(priv = (struct aliasfile_private *)dp->private)) {
 	priv = &aliasfile_template;
+    }
 
     DEBUG(DBG_DRIVER_HI, "dtd_aliasfile called\n");
     if (priv->database == NULL) {
@@ -268,72 +269,18 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 	}
     }
 
-    /* If a director owner exists, compute the owner address for this
-    mailing list and add it to each generated address. This is for
-    the benefit of outgoing SMTP, which wants to use it in the
-    envelope for error returns. However, don't waste time if there
-    aren't any addresses to set the owner for! */
-
-    if (!(dp->owner) ||
-        !(owner = expand_string(dp->owner,
-                           (struct addr *)NULL,
-                           dp->default_home,
-                           dp->default_user)) ||
-        !(preparsed_owner = preparse_address(owner, &error)))
-    {
-        owner = NULL;
-    } else {
-        /* Must verify that this owner is sensible (to avoid things like
-	 * xxxx-request-request).  To do this, use the pre-parsed address.
-	 * However, verify may change the value of this address by removing the
-	 * local domain name.  We don't want this, because it might be a
-	 * non-standard local domain name for list handling.  So make a copy
-	 * beforehand.  Don't waste time verifying "postmaster", which should
-	 * always be valid. */
-        
-        owner = copy(preparsed_owner);		/* copy to use later */
-         
-        if (strcmp(preparsed_owner, "postmaster") != 0) { 
-            struct addr *new2 = alloc_addr();
-	    struct addr *defer2 = alloc_addr();
-	    struct addr *fail2_safe = alloc_addr();
-	    struct addr *fail2;
-	    struct addr *okay = NULL;
-	    int was_reopen = (dp->flags & ALIAS_REOPEN);
-
-	    dp->flags &= ~ALIAS_REOPEN;		/* make sure db not closed! needed below! */
-            
-	    fail2 = fail2_safe;			/* verify_addr_list can clobber last arg */
-	    new2->in_addr = preparsed_owner;
-	    new2->work_addr = preparsed_owner;
-            
-	    /* disable use of smartuser driver */
-	    new2->flags = ADDR_SMARTUSER;
-            
-	    verify_addr_list(new2, &okay, &defer2, &fail2);
-	    if (!okay) {
-		xfree(owner);
-		owner = NULL;
-	    }
-	    xfree(new2);
-	    xfree(defer2);
-	    xfree(fail2_safe);
-	    if (was_reopen) {
-		dp->flags |= ALIAS_REOPEN;
-	    }
-        }     
-    }
-
     /* Set up the new addresses */
 
     for (cur = in; cur; cur = next) {
-	char *value;			/* value from lookup_database */
+	char *vret;			/* value from lookup_database */
+	char *value;			/* copy of vret */
 	struct addr *list;		/* list returned by lookup */
 	int ret;
 
 	next = cur->succ;
 
-	ret = lookup_database(priv->database, cur->remainder, &value, &error);
+	ret = lookup_database(priv->database, cur->remainder, &vret, &error);
+
 	switch (ret) {
 	    struct addr *next_list;
 
@@ -366,32 +313,94 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 	     *      later queue run will hopefully take care of the
 	     *      problem.
 	     */
-	    DEBUG3(DBG_DRIVER_LO, "director %s: lookup failure on %s: %s\n",
-		   dp->name, cur->remainder, error);
 	    error = xprintf("director %s: lookup failure on %s: %s",
 			    dp->name, cur->remainder, error);
-	    cur->error =
-		note_error(ret == DB_AGAIN || ret == FILE_AGAIN?
-				ERR_CONFERR|ERR_158:
-				ERR_158,
-			   error);
+	    DEBUG1(DBG_DRIVER_LO, "dtd_aliasfile: %s\n", error);
+	    cur->error = note_error((ret == DB_AGAIN || ret == FILE_AGAIN) ? (ERR_CONFERR | ERR_158) : ERR_158, error);
 	    cur->succ = *defer;
 	    *defer = cur;
 	    break;
 
 	case DB_SUCCEED:
-            DEBUG4(DBG_DRIVER_LO,
-		   "director %s: matched %s, aliased to %s, \n  owner %s\n",
-		   dp->name, cur->remainder, value,
-		   owner? owner : "(none)");
-
 	    cur->director = dp;		/* set the director which matched */
 
 	    /* get the addr structures */
 	    list = NULL;
 	    error = NULL;
+	    value = COPY_STRING(vret);	/* verifying the owner may call lookup_database() again */
+
 	    (void) process_field((char *)NULL, value, (char *)NULL,
 				 (char *)NULL, &list, F_ALIAS, &error);
+	    /*
+	     * If a director owner exists, compute the owner address for this
+	     * mailing list and add it to each generated address.  This is for
+	     * the benefit of outgoing SMTP, which wants to use it in the
+	     * envelope for error returns.  However, don't waste time if there
+	     * aren't any addresses to set the owner for!
+	     */
+	    if (!(dp->owner) ||
+		!(owner = expand_string(dp->owner,
+					(struct addr *) NULL,
+					dp->default_home,
+					cur->remainder)) ||
+		!(preparsed_owner = preparse_address(owner, &error)))
+	    {
+		DEBUG1(DBG_DRIVER_HI, "dtd_aliasfile: no owner set for %s.\n", value);
+		owner = NULL;
+	    } else {
+		/* 
+		 * Must verify that this owner is sensible (to avoid things
+		 * like xxxx-request-request).  To do this, use the pre-parsed
+		 * address.  However, verify may change the value of this
+		 * address by removing the local domain name.  We don't want
+		 * this, because it might be a non-standard local domain name
+		 * for list handling.  So make a copy beforehand.  Don't waste
+		 * time verifying "postmaster", which should always be valid.
+		 */
+		owner = copy(preparsed_owner); /* copy to use later */
+         
+		if (strcmpic(preparsed_owner, "postmaster") != 0) { 
+		    struct addr *new2 = alloc_addr();
+		    struct addr *defer2 = alloc_addr();
+		    struct addr *fail2_safe = alloc_addr();
+		    struct addr *fail2;
+		    struct addr *okay = NULL;
+		    int was_reopen = (dp->flags & ALIAS_REOPEN);
+
+		    DEBUG1(DBG_DRIVER_MID, "dtd_aliasfile: owner is not postmaster (is '%s'), will attempt to verify.\n", preparsed_owner);
+
+		    dp->flags &= ~ALIAS_REOPEN;	/* make sure db not closed! needed again! */
+            
+		    fail2 = fail2_safe;	/* verify_addr_list can clobber its
+					 * last arg and we need to save it so
+					 * we can free it when we're done
+					 */
+		    new2->in_addr = preparsed_owner;
+		    new2->work_addr = preparsed_owner;
+            
+		    /* disable use of smartuser driver */
+		    new2->flags = ADDR_SMARTUSER;
+            
+		    verify_addr_list(new2, &okay, &defer2, &fail2);
+		    if (!okay) {
+			xfree(owner);
+			owner = NULL;
+		    }
+		    xfree(new2);
+		    xfree(defer2);
+		    xfree(fail2_safe);
+		    if (was_reopen) {
+			dp->flags |= ALIAS_REOPEN;
+		    }
+		}     
+	    }
+            DEBUG4(DBG_DRIVER_LO,
+		   "director %s: matched %s, aliased to %s, \n  owner %s\n",
+		   dp->name, cur->remainder, value,
+		   owner ? owner : "(none)");
+	    /*
+	     * Now we can check the process_field results...
+	     */
 	    if (error) {
 		/*
 		 * ERR_113 - alias parsing error
@@ -459,7 +468,7 @@ dtv_aliasfile(dp, in, retry, okay, defer, fail)
     register struct addr *cur;		/* temp for processing input */
     struct addr *next;			/* next addr to process */
     struct aliasfile_private *priv;	/* private data area */
-    char *error;			/* error message */
+    char *error = NULL;			/* error message */
 #ifdef USE_TARGET_DOMAIN
     char *tdbname = NULL;
 #endif
