@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_102:altuser.c,v 1.5 1997/07/01 01:33:13 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_103:altuser.c,v 1.6 1998/10/07 05:38:37 woods Exp"
  */
 
 /*
@@ -163,7 +163,7 @@ dtd_altuser(dp, in, out, new, defer, fail)
     int altuser_uid;			/* uid for altuser entry */
     int altuser_gid;			/* gid for altuser entry */
     char * altuser_home;		/* home for altuser entry */
-    char *error;			/* error message */
+    char *error = NULL;			/* error message */
 #ifdef USE_TARGET_DOMAIN
     char *tdbname = NULL;
 #endif
@@ -430,7 +430,7 @@ dtv_altuser(dp, in, retry, okay, defer, fail)
     register struct addr *cur;		/* temp for processing input */
     struct addr *next;			/* next addr to process */
     struct altuser_private *priv;	/* private data area */
-    char *error;			/* error message */
+    char *error = NULL;			/* error message */
 #ifdef USE_TARGET_DOMAIN
     char *tdbname = NULL;
 #endif
