@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_102:fwdfile.c,v 1.26 1998/04/11 23:31:23 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_103:fwdfile.c,v 1.28 1998/10/07 05:38:37 woods Exp"
  */
 
 /*
@@ -407,7 +407,7 @@ finish_forward(dp, addr, new, defer, fail)
     struct addr *cur;			/* addrs from process_field */
     struct addr *next;			/* next addr from process_field */
     int flags = ADDR_FWDTYPE;		/* addr structure flags value */
-    char *error;			/* error message */
+    char *error = NULL;			/* error message */
     struct str fwdtobuf;
     char *remainder = addr->remainder;
     int c_save = 0;				/* saved start of suffix */
@@ -686,7 +686,7 @@ finish_forward(dp, addr, new, defer, fail)
 
     DEBUG3(DBG_DRIVER_LO, "director <%s> matched <%s> owner <%s>\n",
            dp->name, addr->remainder,
-           owner? owner : "none");
+           owner? owner : "(none)");
     xfree(fn);
     return SUCCEED;
 }
