@@ -1,4 +1,6 @@
-/* @(#) smartuser.c,v 1.9 1997/03/07 03:23:43 woods Exp */
+/*
+#ident	"@(#)smail/directors:RELEASE-3_2_0_103:smartuser.c,v 1.10 1998/10/07 05:38:38 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -92,7 +94,7 @@ dtd_smartuser(dp, in, out, new, defer, fail)
     register struct addr *cur;		/* temp for processing input */
     struct addr *next;			/* next value for cur */
     struct addr *pass = NULL;		/* addrs to pass to next director */
-    char *error;
+    char *error = NULL;
     char *temp;
     struct smartuser_private *priv;
 
