@@ -1,4 +1,6 @@
-/* @(#) error.h,v 1.15 1996/05/29 18:48:23 woods Exp */
+/*
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:error.h,v 1.16 1998/09/27 00:38:56 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -100,3 +102,4 @@
 #define ERR_183		183L		/* reroute file lookup error */
 #define ERR_184		184L		/* resolve timeout - too long in directors/routers */
 #define ERR_185		185L		/* altuser passwd file is unparsable */
+#define ERR_186		186L		/* canonical host not found */
