@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:extern.h,v 1.62 1998/02/15 21:12:33 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:extern.h,v 1.65 1998/10/07 05:25:53 woods Exp"
  */
 
 /*
@@ -94,6 +94,7 @@ extern int smtp_accept_max;		/* max simultaneous SMTPs to accept */
 extern int smtp_accept_queue;		/* simultaneous SMTPs until queueonly */
 extern int smtp_debug;			/* allow DEBUG command in SMTP */
 extern int smtp_info;			/* allow EXPN/VRFY commands in SMTP */
+extern char *smtp_recipient_no_verify;	/* ip list for not verifying RCPT TO  */
 extern char *smtp_remote_allow;		/* ip list that can send remote smtp */
 extern char *smtp_local_net;		/* the local network calculated from src. addr */
 extern int smtp_hello_verify;		/* verify HELO/EHLO host has A RR matching sockaddr */
@@ -171,6 +172,7 @@ extern enum prog_type prog_type;	/* type of program we are running as */
 #endif	/* MAIN_H */
 extern int me_too;			/* sender allowed in aliases */
 extern struct addr *recipients;		/* list of recipient addresses */
+extern int num_recipients;		/* number of entries in 'recipients' */
 extern char *primary_name;		/* primary local name from hostnames */
 extern FILE *errfile;			/* file to write debug messages to */
 extern int real_uid;			/* saved real uid before ruid setup */
@@ -277,6 +279,7 @@ extern void free_block();
 
 #ifdef HAVE_BIND
 /* external functions defined in bindlib.c */
+extern int bind_check_if_canonical_host();
 extern int bind_check_if_local_mxs();
 extern int bind_addr();
 extern char * bind_compute_domain();
