@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:lookup.c,v 1.34 1997/05/12 16:38:05 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:lookup.c,v 1.36 1998/09/27 00:42:02 woods Exp"
  */
 
 /*
@@ -227,6 +227,8 @@ open_database(name, proto, retries, interval, statp, db, error)
 {
     register struct proto *pp;
 
+    DEBUG2(DBG_DRIVER_HI, "open_database: db name = %s, db proto = %s.\n", name, proto);
+
     for (pp = protos; proto && pp < end_protos; pp++) {
 	if (EQ(proto, pp->proto)) {
 	    /* found the requested access method */
@@ -274,15 +276,27 @@ close_database(priv)
  *		to be possible.
  */
 int
-lookup_database(db, key, value, error)
+lookup_database(db, key, valuep, errorp)
     char *db;				/* open database */
     char *key;				/* search key */
-    char **value;			/* store value here */
-    char **error;			/* store error message here */
+    char **valuep;			/* store value here */
+    char **errorp;			/* store error message here */
 {
     register struct generic_db *gdb = (struct generic_db *)db;
+    register int ret;
 
-    return (*gdb->proto->lookup)(gdb, key, value, error);
+    DEBUG1(DBG_DRIVER_HI, "lookup_database(): looking for <%s>.\n", key);
+
+    *valuep = (char *) NULL;
+    ret = (*gdb->proto->lookup)(gdb, key, valuep, errorp);
+
+    DEBUG4(DBG_DRIVER_HI,
+	   "lookup_database(): returning %d, after looking for <%s>, found value = <%s>, error = %s.\n",
+	   ret, key,
+	   *valuep ? *valuep : "(none)",
+	   *errorp ? *errorp : "(none)");
+
+    return ret;
 }
 
 /*
@@ -315,6 +329,8 @@ bsearch_open(name, proto, retries, interval, statp, db, error)
     register struct bsearch_db *priv;
     register FILE *f;
 
+    DEBUG1(DBG_DRIVER_HI, "bsearch_open: called to open %s.\n", name);
+
     name = make_lib_fn(name);
     if (name == NULL) {
 	*error = "No directory for file";
@@ -367,6 +383,8 @@ static void
 bsearch_close(db)
     struct bsearch_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "bsearch_close: called to close %s.\n", db->name);
+
     (void) fclose(db->f);
     xfree(db->name);
     xfree((char *)db);
@@ -394,7 +412,7 @@ bsearch_lookup(db, key, value, error)
     static int str_inited = FALSE;	/* TRUE if str has been STR_INIT'd */
     int i;				/* temp */
 
-    DEBUG1(DBG_DRIVER_HI, "bsearch_lookup: looking for <%s>\n", key);
+    DEBUG2(DBG_DRIVER_HI, "bsearch_lookup: looking for <%s> in %s.\n", key, db->name);
 
     if (!str_inited) {
 	/*
@@ -505,6 +523,8 @@ lsearch_open(name, proto, retries, interval, statp, db, error)
     register struct lsearch_db *priv;
     register FILE *f;
 
+    DEBUG1(DBG_DRIVER_HI, "lsearch_open: called to open %s.\n", name);
+
     name = make_lib_fn(name);
     if (name == NULL) {
 	*error = "No directory for file";
@@ -553,6 +573,8 @@ static void
 lsearch_close(db)
     struct lsearch_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "lsearch_close: called to close %s.\n", db->name);
+
     (void) fclose(db->f);
     xfree(db->name);
     xfree((char *)db);
@@ -668,7 +690,7 @@ lsearch_lookup(db, key, value, error)
     register char *entry;		/* entry from read_entry() */
     register char *p;			/* temp */
 
-    DEBUG1(DBG_DRIVER_HI, "lsearch_lookup: looking for <%s>\n", key);
+    DEBUG2(DBG_DRIVER_HI, "lsearch_lookup: looking for <%s> in %s.\n", key, db->name);
 
     /* always start from the beginning of the file */
     (void) fseek(db->f, 0L, 0);
@@ -782,6 +804,8 @@ dbmbase_open(name, proto, retries, interval, statp, db, error)
     char *pag_file;			/* name of .pag file */
     register struct dbmbase_db *priv;
 
+    DEBUG1(DBG_DRIVER_HI, "dbmbase_open: called to open %s.\n", name);
+
     name = make_lib_fn(name);
     if (name == NULL) {
 	*error = "No directory for file";
@@ -867,6 +891,8 @@ static void
 dbmbase_close(db)
     struct dbmbase_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "dbmbase_close: called to close %s.\n", db->name);
+
 #ifdef HAVE_NDBM
     (void) dbm_close(db->db);
 #else
@@ -893,7 +919,6 @@ dbmbase_lookup(db, key, value, error)
     static int temp_size;		/* size of temp_key area */
     static char *temp_data = NULL;	/* growable temp area */
     int len;				/* length of key */
-    register char *p;
 
 #ifndef HAVE_NDBM
     if (opendb != db) {
@@ -901,15 +926,18 @@ dbmbase_lookup(db, key, value, error)
 	    dbmclose();
 	    opendb = NULL;
 	}
-	if (dbminit(db->name) < 0)
-	    return FAIL;
+	if (dbminit(db->name) < 0) {
+	    *error = xprintf("dbminit(): failed to initialize %s: %s.", db->name, strerror(errno));
+	    DEBUG1(DBG_DRIVER_LO, "dbmbase_lookup: dbminit(): %s.\n", *error);
+	    return FILE_FAIL;
+	}
     }
 #endif
 
     /*
-     * convert the key to lower case, as fetch() is case-sensitive
-     * for efficiency, keep around the malloc'd region used to store
-     * the down-cased key.
+     * convert the key to lower case, as (dbm_)fetch() is case-sensitive for
+     * efficiency, keep around the malloc'd region used to store the down-cased
+     * key.
      */
     len = strlen(key) + 1;
     if (temp_data == NULL) {
@@ -919,17 +947,18 @@ dbmbase_lookup(db, key, value, error)
 	temp_size = len;
 	temp_data = xrealloc(temp_data, temp_size);
     }
-    for (p = temp_data; *key; p++, key++) {
-	*p = lowercase(*key);
-    }
-    *p = '\0';
+    strcpy(temp_data, key);
+    str2lower(temp_data);
+
+    DEBUG2(DBG_DRIVER_HI, "dbmbase_lookup: looking for <%s> in %s.\n", temp_data, db->name);
 
     the_key.dptr = temp_data;
     the_key.dsize = len;
 #ifdef HAVE_NDBM
     the_value = dbm_fetch(db->db, the_key);
     if (dbm_error(db->db)) {
-	*error = xprintf("Error in fetching %s", the_key);
+	*error = xprintf("Error in fetching %s: %s", temp_data, strerror(errno));
+	DEBUG1(DBG_DRIVER_LO, "dbmbase_lookup: dbm_error(): %s.\n", *error);
 	return FILE_FAIL;
     }
 #else
@@ -943,8 +972,14 @@ dbmbase_lookup(db, key, value, error)
 	}
 	(void) strcpy(temp_data, the_value.dptr);
 	*value = temp_data;
+
+	DEBUG1(DBG_DRIVER_HI, "dbmbase_lookup: return <%s>\n", *value);
+
 	return DB_SUCCEED;
     }
+
+    DEBUG1(DBG_DRIVER_HI, "dbmbase_lookup: did not find <%s>\n", temp_data);
+
     return DB_NOMATCH;
 }
 
@@ -1005,6 +1040,8 @@ aliasyp_open(name, proto, retries, interval, statp, db, error)
     char **db;				/* store open database info here */
     char **error;			/* store error message here */
 {
+    DEBUG1(DBG_DRIVER_HI, "aliasyp_open: called to open %s.\n", name);
+
     return yp_open(name, proto, retries, interval, statp, db, error);
 }
 
@@ -1124,6 +1161,8 @@ static void
 aliasyp_close(db)
     struct yp_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "aliasyp_close: called to close %s.\n", db->name);
+
     yp_close(db);
 }
 
@@ -1149,6 +1188,8 @@ yp_lookup(db, key, value, error)
     char **value;			/* return value here */
     char **error;			/* return error message here */
 {
+    DEBUG2(DBG_DRIVER_HI, "yp_lookup: looking for <%s> in %s.\n", key, db->name);
+
     return common_yp_lookup(db, key, value, error, FALSE);
 }
 
@@ -1159,6 +1200,8 @@ aliasyp_lookup(db, key, value, error)
     char **value;			/* return value here */
     char **error;			/* return error message here */
 {
+    DEBUG2(DBG_DRIVER_HI, "aliasyp_lookup: looking for <%s> in %s.\n", key, db->name);
+
     return common_yp_lookup(db, key, value, error, TRUE);
 }
 
@@ -1307,6 +1350,8 @@ aliasni_open(name, proto, retries, interval, statp, db, error)
 {
     register struct aliasni_db *priv;
 
+    DEBUG1(DBG_DRIVER_HI, "aliasni_open: called to open %s.\n", name);
+
     /* build the private area */
 
     alias_setent();
@@ -1331,6 +1376,8 @@ static void
 aliasni_close(db)
     struct aliasni_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "aliasni_close: called to close %s.\n", db->name);
+
     alias_endent();
     xfree(db->expansion_area);
     xfree((char *)db);
@@ -1350,7 +1397,7 @@ aliasni_lookup(db, key, value, error)
     int i,n;
     char        *p;
 
-    DEBUG1(DBG_DRIVER_HI, "aliasni_lookup: looking for <%s>\n",key);
+    DEBUG1(DBG_DRIVER_HI, "aliasni_lookup: looking for <%s> in %s.\n", key, db->name);
 
     a = alias_getbyname(key);
     if ((!a) || (a->alias_members_len == 0)) {
@@ -1442,6 +1489,8 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
     int				flags;
     char			*domain;
 
+    DEBUG1(DBG_DRIVER_HI, "nisplus_open: called to open %s.\n", name);
+
     priv = (struct nisplus_db *)xmalloc(sizeof(*priv));
     priv->proto = proto;
     priv->fieldnum = 1;
@@ -1580,6 +1629,8 @@ static void
 nisplus_close(db)
     struct nisplus_db *db;
 {
+    DEBUG1(DBG_DRIVER_HI, "nisplus_close: called to close %s.\n", db->name);
+
     xfree(db->name);
     xfree((char *)db);
     return;
@@ -1609,9 +1660,9 @@ nisplus_lookup(db, key, value, error)
     /* make the name to look up */
     /* prepend the search string as a "subdomain" of the table */
     name = (char*) xmalloc(strlen(key)+strlen(db->table)+2);
-    sprintf(name,db->table,key);
+    sprintf(name, db->table, key);
 
-    DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: looking for <%s>\n", key);
+    DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: looking for <%s> in %s.\n", key, db->name);
     DEBUG1(DBG_DRIVER_HI, "nisplus_lookup: asking NIS+ for <%s>\n", name);
 
     /* look up the object */
@@ -1736,7 +1787,7 @@ nisplus_lookup(db, key, value, error)
 	/* sleep interval for retry */
 	if (retries > 0) {
 	    DEBUG2(DBG_DRIVER_HI,
-		   "bsearch_lookup: will retry open(%s) after %d seconds\n",
+		   "nisplus_lookup: will retry open(%s) after %d seconds\n",
 		   name, db->interval);
 	    sleep(db->interval);
 	}
