@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:main.c,v 1.48 1998/04/13 03:33:39 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:main.c,v 1.50 1998/10/01 05:34:06 woods Exp"
  */
 
 /*
@@ -86,9 +86,11 @@ enum dot_usage dot_usage;		/* how do we treat . on input */
 enum deliver_mode
     deliver_mode = DELIVER_DEFAULT;	/* foreground, background or queued */
 struct addr *recipients;		/* list of recipient addresses */
+int num_recipients = 0;			/* number of entries in 'recipients' */
 char *primary_name;			/* primary local name from hostnames */
 FILE *errfile = NULL;			/* file to write debug messages to */
 int real_uid;				/* saved real uid before ruid setup */
+int real_gid;				/* saved real gid before rgid setup */
 enum prog_type prog_type;		/* type of program we are running as */
 char **save_argv;			/* saved pointer to arguments */
 int some_deferred_addrs;		/* don't unlink spool file */
@@ -357,8 +359,7 @@ main(argc, argv)
     if (config_file != save_config_file || arg_second_config_file ||
 	arg_director_file || arg_router_file || arg_transport_file ||
 	arg_qualify_file || arg_retry_file || arg_smail_lib_dir ||
-	arg_smail_util_dir || arg_alias_file ||
-	operation_mode == REBUILD_ALIASES)
+	arg_smail_util_dir || arg_alias_file)
     {
 	/*
 	 * a config_file was set, or unset from the command args
@@ -540,10 +541,15 @@ main(argc, argv)
      * sends mail.
      */
     real_uid = (int) getuid();
+    real_gid = (int) getgid();
     prog_euid = (int) geteuid();		/* keep a copy of the effictive id's */
     prog_egid = (int) getegid();
-    setuid(prog_euid);
-    setgid(prog_egid);
+    if (prog_egid != real_gid) {
+	setgid(prog_egid);
+    }
+    if (prog_euid != real_uid) {
+	setuid(prog_euid);
+    }
 
     /*
      * If the current effective UID does not match the required ID,
@@ -568,7 +574,7 @@ main(argc, argv)
      */
 
     if (arg_debug_file) {
-	if (!(new_errfile = fopen_as_user(arg_debug_file, "a", real_uid, (int) getgid(), 0600))) {
+	if (!(new_errfile = fopen_as_user(arg_debug_file, "a", real_uid, real_gid, 0600))) {
 	    write_log(WRITE_LOG_TTY, "Warning: Cannot open debug file %s: %s\n",
 		      arg_debug_file, strerror(errno));
 	    arg_debug_file = NULL;
@@ -728,7 +734,8 @@ main(argc, argv)
 	print_variables();
 	break;
 
-    case REBUILD_ALIASES:
+    case REBUILD_ALIASES: {
+	int status;
 
 	if (smail_util_dir == NULL) {
 	    if (errfile) {
@@ -756,7 +763,7 @@ main(argc, argv)
 	}
 	DEBUG4(DBG_MAIN_LO, "main: about to run %s %s %s %s\n",
 	       utilargs[0], utilargs[1], utilargs[2] ? utilargs[2] : "", utilargs[3] ? utilargs[3] : "");
-	child = open_child(utilargs, (char **) NULL, stdin, stdout, fileno(stderr),
+	child = open_child(utilargs, (char **) NULL, (FILE **) NULL, (FILE **) NULL, fileno(stderr),
 			   CHILD_MINENV, (int) getuid(), (int) getgid());
 	if (child == -1) {
 	    if (errfile) {
@@ -765,10 +772,26 @@ main(argc, argv)
 	    }
 	    exit(EX_UNAVAILABLE);
 	}
-	close_child((FILE *)NULL, (FILE *)NULL, child);
+	if ((status = close_child((FILE *) NULL, (FILE *) NULL, child)) != 0) {
+	    if (status == -1) {
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: failed to reap child process from %s: %s.",
+			  program, utilargs[0], strerror(errno));
+		exit(EX_OSERR);
+	    } else if (status & 0xff) {
+		char *errstr = xprintf("%s: %s killed by signal %d",
+				       program, utilargs[0], status & 0x7f);
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, errstr);
+		exit(EX_UNAVAILABLE);
+	    } else {
+		char *errstr = xprintf("%s: %s returned status %s (%d)",
+				       program, utilargs[0], strsysexit(status>>8), status>>8);
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, errstr);
+		exit(EX_DATAERR);
+	    }
+	}
 	exit(0);
 	/*NOTREACHED*/
-
+    }
     default:
 	if (errfile) {
 	    (void) fprintf(errfile, "%s: option not supported\n", program);
@@ -795,6 +818,9 @@ main(argc, argv)
 
 /*
  * initialize_state - set some parameters to their default value
+ * 
+ * NOTE:  This routine may be called at least twice in the life of any given
+ * smail process.
  */
 void
 initialize_state()
@@ -817,6 +843,7 @@ initialize_state()
     me_too = FALSE;
     error_processing = ERROR_DEFAULT;
     recipients = NULL;
+    num_recipients = 0;
     some_deferred_addrs = FALSE;
     call_defer_message = FALSE;
     sender_is_trusted = TRUE;
@@ -1456,6 +1483,7 @@ process_args(args)
 	    cur->gid = nobody_gid;
 	    cur->succ = recipients;
 	    recipients = cur;
+	    num_recipients++;
 	}
     }
 }
