@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:modes.c,v 1.75 1998/04/11 23:30:35 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:modes.c,v 1.77 1998/09/27 00:34:18 woods Exp"
  */
 
 /*
@@ -158,6 +158,7 @@ extern int errno;
 static int start_daemon();
 static void bg_run_queue();
 static void do_run_queue();
+static void process_one_spool_file();
 static void sig_unlink();
 static void sig_close();
 static void set_queue_only();
@@ -1567,9 +1568,10 @@ static void
 bg_run_queue(ls)
     int ls;		/* if >=0, close this descriptor in child */
 {
-    int pid = fork();
+    int pid;
 
-    if (pid == 0) {
+    switch ((pid = fork())) {
+    case 0:				/* child */
 	if (ls >= 0)
 	    close(ls);
 #ifdef POSIX_OS
@@ -1584,6 +1586,12 @@ bg_run_queue(ls)
 	(void) alarm(0);
 	do_run_queue();
 	exit(0);
+	/* NOTREACHED */
+    case -1:				/* error */
+	panic(EX_OSERR, "fork() failed when trying to start background queue run daemon.");
+	/* NOTREACHED */
+    default:				/* parent */
+	    break;
     }
 }
 
@@ -1734,6 +1742,26 @@ do_run_queue()
 
     DEBUG(DBG_MAIN_MID, "do_run_queue: called\n");
 
+    if (num_recipients == 1 && recipients) {
+	/*
+	 * If it wasn't for the fact that there might be more than one spool
+	 * directory we could avoid reading the whole queue just to see if this
+	 * one spool file exists....
+	 */
+	work = scan_spool_dirs();
+	if (! *work) {
+	    if (errfile) {
+		(void)fprintf(errfile,
+			      "%s: scan_spool_dirs() could not find spool file for: %s\n",
+			      program, recipients->in_addr);
+		(void)fflush(errfile);
+	    }
+	}
+	process_one_spool_file(*work);
+	DEBUG(DBG_MAIN_HI, "do_run_queue: finished processing one queue entry\n");
+        return;
+    }
+
     /* get work, and do it with child processes */
     work = scan_spool_dirs();
     while (*work) {
@@ -2005,6 +2033,7 @@ do_smtp(in, out)
 	    pid = fork_wait();
 	    if (pid < 0) {
 		/* can't fork(), try again later for all messages */
+		DEBUG1(DBG_MAIN_LO, "do_smtp(): fork_wait() failed: %s.\n", strerror(errno));
 		if (errfile) {
 		    (void)fprintf(errfile,
 				  "%s: fork() failed: %s, try again later\n",
@@ -2083,13 +2112,14 @@ process_spool_file(spfn)
     int pid;
     struct stat statbuf;
 
+    DEBUG1(DBG_MAIN_MID, "process_spool_file(%s): called\n", spfn);
     /*
      * Verify the spool file exists before we try to process it
      */
     if (stat(spfn, &statbuf) < 0) {
-	write_log(WRITE_LOG_SYS, "process_spool_file: %s: File possibly already processed: %s",
-		  spfn, strerror(errno));
-	return FAIL;
+	DEBUG2(DBG_MAIN_MID, "process_spool_file: %s: File possibly already processed: %s",
+	      spfn, strerror(errno));
+	return SUCCEED;			/* we only fail if fork_wait() fails!  */
     }
 
     /*
@@ -2097,6 +2127,7 @@ process_spool_file(spfn)
      */
     if ((pid = fork_wait()) < 0) {
 	/* can't fork(), try again later */
+	DEBUG1(DBG_MAIN_LO, "process_spool_file: fork_wait() failed: %s.\n", strerror(errno));
 	if (errfile) {
 	    (void)fprintf(errfile,
 			  "%s: fork() failed: %s, try again later\n",
@@ -2106,51 +2137,65 @@ process_spool_file(spfn)
 	return FAIL;
     }
     if (pid == 0) {
-	/* in child process */
-	char **argv;		/* arguments from spool file */
-
-	/* message grade is encoded in the last char of the filename */
-	msg_grade = spfn[strlen(spfn) - 1];
-
-	/* initialize state before reading state from the spool file */
-	initialize_state();
-
-	/* in child process, open the message and attempt delivery */
-	if (open_spool(spfn, TRUE, WRITE_LOG_SYS) == FAIL) {
-	    if (exitvalue == EX_TEMPFAIL || exitvalue == EX_NOINPUT) {
-		/* XXX in theory this should be just a DEBUG() */
-		write_log(WRITE_LOG_TTY|WRITE_LOG_SYS,
-			  "process_spool_file: %s/%s: file %s",
-			  spool_dir, input_spool_fn,
-			  (exitvalue == EX_TEMPFAIL) ? "already locked; skipping"
-						     : "not found (already processed?)");
-	    }
-	    exit(exitvalue);
-	}
-	argv = read_message();
-
-	if (argv == NULL) {
-	    panic(EX_OSFILE, "failed to read queued message in %s/%s",
-		  spool_dir, input_spool_fn);
-	    /*NOTREACHED*/
-	}
-
-	/* process arguments from the spool file */
-	process_args(argv);
-
-	/* perform delivery */
-	deliver_mail();
+	process_one_spool_file(spfn);
 
 	/* close the sytem-wide log files */
 	close_system_logs();
 
-	/* all done with the message */
 	exit(exitvalue);
+	/* NOREACHED */
     }
 
     return SUCCEED;
 }
 
+/*
+ * process_one_spool_file - open, read, and process a spool file
+ *
+ * return FAIL if anything goes wrong, else return SUCCEED.
+ */
+static void
+process_one_spool_file(spfn)
+    char *spfn;
+{
+    /* in child process */
+    char **argv;		/* arguments from spool file */
+
+    /* message grade is encoded in the last char of the filename */
+    msg_grade = spfn[strlen(spfn) - 1];
+
+    /* initialize state before reading state from the spool file */
+    initialize_state();			/* XXX probably second call */
+
+    /* attempt to open the message */
+    if (open_spool(spfn, TRUE, WRITE_LOG_SYS) == FAIL) {
+	if (exitvalue == EX_TEMPFAIL || exitvalue == EX_NOINPUT) {
+	    DEBUG2(DBG_MAIN_LO, "process_one_spool_file: %s: file %s", spfn,
+		   (exitvalue == EX_TEMPFAIL) ? "already locked; skipping" : "not found (already processed?)");
+	} else {
+	    panic(exitvalue, "process_one_spool_file(): could not process %s -- see log.", spfn);
+	    /* NOTREACHED */
+	}
+	return;
+    }
+
+    /* read the spool file header to get the delivery args */
+    argv = read_message();
+    if (argv == NULL) {
+	panic(EX_OSFILE, "failed to read queued message in %s", spfn);
+	/* NOTREACHED */
+    }
+
+    /* process arguments from the spool file */
+    process_args(argv);
+
+    /* perform delivery */
+    deliver_mail();
+
+    /* all done with the message */
+    return;
+}
+
 /*
  * fork_wait - fork and have the parent wait for the child to complete
  *
@@ -2162,17 +2207,21 @@ process_spool_file(spfn)
 int
 fork_wait()
 {
-    int pid = fork();
+    int pid;
     int i;
 
-    if (pid == 0) {
+    switch ((pid = fork())) {
+    case 0:
 	return 0;
-    }
-    if (pid < 0) {
-	return -1;
-    }
-    while ((i = wait((STATUS_TYPE *)0)) >= 0 && i != pid) ;
 
+    case -1:
+	return -1;
+
+    default:
+	while ((i = wait((STATUS_TYPE *) NULL)) >= 0 && i != pid)
+	    ;
+	break;
+    }
     return pid;
 }
 
