@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:smtprecv.c,v 1.72 1998/06/08 01:02:07 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:smtprecv.c,v 1.77 1998/10/08 06:13:33 woods Exp"
  */
 
 /*
@@ -96,7 +96,7 @@ static enum e_smtp_commands read_smtp_command(FILE *, FILE *);
 static void expand_addr(char *, FILE *);
 static int verify_sender(char *, char *, FILE *);
 static int verify_addr(char *, FILE *, int);
-static char *verify_host(char *, struct sockaddr_in *, struct hostent*, char const **);
+static char *verify_host(char *, struct sockaddr_in *, struct hostent*, char const *, char const **);
 static void smtp_input_signals(void);
 static void smtp_processing_signals(void);
 static void set_term_signal(int);
@@ -165,7 +165,7 @@ char *sender_host_really = NULL;	/* result of PTR lookup */
 #define BS_UNSPECIFIED -1L
 typedef long BodySize;
 
-int num_recipients = 0;			/* number of recipient addresses */
+int num_smtp_recipients = 0;			/* number of recipient addresses */
 
 
 /*
@@ -353,6 +353,18 @@ receive_smtp(in, out)
     }
 #endif /* HAVE_RFC1413 */
 
+    if (out) {
+	write_log(WRITE_LOG_SYS, "remote connection from %s%s%s%s%s%s%s%s%s",
+		  ident_sender ? ident_sender : "",
+		  ident_sender ? "@" : "",
+		  sender_host ? sender_host : "",
+		  sender_host_really ? "(" : "",
+		  sender_host_really ? sender_host_really : "",
+		  sender_host_really ? ")" : "",
+		  sender_host_addr ? " [" : "",
+		  sender_host_addr ? sender_host_addr : "",
+		  sender_host_addr ? "]" : "");
+    }
     while (! term_signal || out == NULL) {
 	char *orig_data = NULL;
 
@@ -406,7 +418,7 @@ receive_smtp(in, out)
 			sender_host_really = COPY_STRING(shp->h_name);
 		    }
 		}
-		if (! (sender_host = verify_host(data, &from_sa, shp, &error))) {
+		if (! (sender_host = verify_host(data, &from_sa, shp, shp ? NULL : hstrerror(h_errno), &error))) {
 		    /* XXX how many MTAs will break if this is multi-lined??? */
 		    fprintf(out, "501 %s requires a valid host name as operand: '%s' rejected from %s%s%s%s%s%s%s%s.\r\n",
 			    ehlo_p ? "EHLO" : "HELO", data,
@@ -500,7 +512,7 @@ receive_smtp(in, out)
 			sender_host_addr ? sender_host_addr : "",
 			sender_host_addr ? "]" : "");
 		fflush(out);
-		write_log(WRITE_LOG_SYS, "remote MAIL FROM: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s%s",
+		write_log(WRITE_LOG_SYS, "remote MAIL FROM: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s",
 			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 			  sender_host ? sender_host : "",
 			  sender_host_really ? "(" : "",
@@ -641,7 +653,7 @@ receive_smtp(in, out)
 			{
 			    if (out) {
 				*(rest+restlen) = '\0';
-				fprintf(out, "555 Unknown MAIL TO: option %s\r\n", rest);
+				fprintf(out, "555 Unknown MAIL FROM: option %s\r\n", rest);
 			    }
 			    goto fail_mail_cmd;
 			}
@@ -685,7 +697,7 @@ receive_smtp(in, out)
 			sender_host_addr ? sender_host_addr : "",
 			sender_host_addr ? "]" : "");
 		fflush(out);
-		write_log(WRITE_LOG_SYS, "remote MAIL TO: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s%s",
+		write_log(WRITE_LOG_SYS, "remote RCPT TO: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s",
 			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 			  sender_host ? sender_host : "",
 			  sender_host_really ? "(" : "",
@@ -709,17 +721,27 @@ receive_smtp(in, out)
 		fflush(out);
 		break;
 	    }
-	    cur = alloc_addr();
 	    if (out) {
-		if (! verify_addr(data, out, 1)) {
+		if (sender_host_addr && smtp_recipient_no_verify &&
+		    match_ip(sender_host_addr, smtp_recipient_no_verify)) {
+		    fprintf(out, "250 '%s' %s%s%s Recipient Unverified.\r\n", orig_data,
+			    (*data == '<') ? "" : "<", data, (*data == '<') ? "" : "<");
+		    fflush(out);
+		    /* fall through to add address to recipient list */
+		} else if (! verify_addr(data, out, 1)) { /* check relay rules */
+		    /* NOTE: error reply is printed by verify_addr() */
 		    break;
 		}
-		if (smtp_max_recipients && (num_recipients++ >= smtp_max_recipients)) {
-		    fprintf(out, "452 Too many recipients.  Administrative limit.\r\n");
+		if (smtp_max_recipients && (num_smtp_recipients++ >= smtp_max_recipients)) {
+		    fprintf(out, "452 Too many recipients.  Administrative limit exceeded.\r\n");
 		    fflush(out);
 		    break;
 		}
 	    }
+	    /*
+	     * create a new address for this recipient and add to the recipients list
+	     */
+	    cur = alloc_addr();
 	    /*
 	     * surround in angle brackets, if the addr begins with `-'.
 	     * This will avoid ambiguities in the data dumped to the spool
@@ -868,7 +890,7 @@ receive_smtp(in, out)
 			sender_host_addr ? sender_host_addr : "",
 			sender_host_addr ? "]" : "");
 		fflush(out);
-		write_log(WRITE_LOG_SYS, "remote EXPN: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s%s",
+		write_log(WRITE_LOG_SYS, "remote EXPN: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s",
 			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 			  sender_host ? sender_host : "",
 			  sender_host_really ? "(" : "",
@@ -933,7 +955,7 @@ receive_smtp(in, out)
 			sender_host_addr ? sender_host_addr : "",
 			sender_host_addr ? "]" : "");
 		fflush(out);
-		write_log(WRITE_LOG_SYS, "remote DEBUG: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s%s",
+		write_log(WRITE_LOG_SYS, "remote DEBUG: '%s' refusing SMTP connection from %s%s%s%s%s%s%s%s%s",
 			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 			  sender_host ? sender_host : "",
 			  sender_host_really ? "(" : "",
@@ -967,6 +989,15 @@ receive_smtp(in, out)
 		    } else {
 			DEBUG(DBG_REMOTE_LO,
 			      "debugging output grabbed by SMTP\r\n");
+			write_log(WRITE_LOG_SYS, "remote DEBUG: '%s' debugging output at level %d grabbed by %s%s%s%s%s%s%s%s%s",
+				  data, temp, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+				  sender_host ? sender_host : "",
+				  sender_host_really ? "(" : "",
+				  sender_host_really ? sender_host_really : "",
+				  sender_host_really ? ")" : "",
+				  sender_host_addr ? " [" : "",
+				  sender_host_addr ? sender_host_addr : "",
+				  sender_host_addr ? "]" : "");
 			fprintf(out, "250 Debugging level: %d\r\n", temp);
 		    }
 		    fflush(out);
@@ -975,6 +1006,15 @@ receive_smtp(in, out)
 		    break;
 		}
 #endif	/* !NODEBUG */
+		write_log(WRITE_LOG_SYS, "remote DEBUG: '%s' debugging at level %d refused to %s%s%s%s%s%s%s%s%s",
+			  data, temp, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "",
+			  sender_host_really ? "(" : "",
+			  sender_host_really ? sender_host_really : "",
+			  sender_host_really ? ")" : "",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
 		fprintf(out, "500 I hear you knocking, but you can't come in\r\n");
 		fflush(out);
 	    }
@@ -982,6 +1022,15 @@ receive_smtp(in, out)
 
 	case HELP_CMD:
 	    if (out) {
+		write_log(WRITE_LOG_SYS, "remote HELP: '%s' requested by %s%s%s%s%s%s%s%s%s",
+			  data, ident_sender ? ident_sender : "", ident_sender ? "@" : "",
+			  sender_host ? sender_host : "",
+			  sender_host_really ? "(" : "",
+			  sender_host_really ? sender_host_really : "",
+			  sender_host_really ? ")" : "",
+			  sender_host_addr ? " [" : "",
+			  sender_host_addr ? sender_host_addr : "",
+			  sender_host_addr ? "]" : "");
 		for (i = 0; i < TABLESIZE(help_msg); i++) {
 		    fprintf(out, "%s\r\n", help_msg[i]);
 		}
@@ -1042,7 +1091,7 @@ reset_state()
 	xfree((char *)cur);
     }
     recipients = NULL;
-    num_recipients = 0;
+    num_smtp_recipients = 0;
 
     /* FYI, do not zap sender_host_addr or any ident stuff, or any other TCP
      * connection related stuff
@@ -1263,7 +1312,7 @@ verify_sender(pp_sender, raw_sender, out)
 		  vaddr->in_addr, vaddr->target,
 		  (form == RFC_ROUTE) ? "RFC_ROUTE" :
 			(form == RFC_ENDROUTE) ? "RFC_ENDROUTE" :
-			(form == UUCP_ROUTE) ? "UCUP_ROUTE" :
+			(form == UUCP_ROUTE) ? "UUCP_ROUTE" :
 			(form == PCT_MAILBOX) ? "PCT_MAILBOX" : "<bad-form!>",
 		  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 		  sender_host,
@@ -1276,8 +1325,8 @@ verify_sender(pp_sender, raw_sender, out)
 	int mxresult;
 	long bindflgs = BIND_DOMAIN_REQUIRED | BIND_DEFER_NO_CONN | BIND_LOCAL_MX_OKAY | BIND_DONT_FILTER;
 	struct rt_info rt_info;
-	static struct error errst = { 0L, NULL };
-	struct error *binderr = &errst;
+	struct error *binderr = NULL;
+	char *binderrmsg;
 	static struct bindlib_private bindpriv =  BIND_TEMPLATE_ATTRIBUTES;
 
 	rt_info.next_host = NULL;
@@ -1291,10 +1340,12 @@ verify_sender(pp_sender, raw_sender, out)
 
 	mxresult = bind_addr(vaddr->target, bindflgs, &bindpriv,
 			     "verify_sender()", &rt_info, &binderr);
+	binderrmsg = (binderr && binderr->message && *binderr->message) ?
+		binderr->message :
+		(mxresult == DB_SUCCEED) ? "(none)" : "unknown error";
 
 	DEBUG4(DBG_REMOTE_MID, "verify_sender(%s): bind_addr(%s): %d, %s.\n",
-	       vaddr->in_addr, vaddr->target, mxresult,
-	       (binderr->message && *binderr->message) ? binderr->message : "no error");
+	       vaddr->in_addr, vaddr->target, mxresult, binderrmsg);
 
 	if (rt_info.next_host) {
 	    xfree(rt_info.next_host);	/* clean up alloc'ed storage we don't need */
@@ -1311,8 +1362,7 @@ verify_sender(pp_sender, raw_sender, out)
 	case FILE_AGAIN:		/* lost contact with server */
 	case FILE_NOMATCH:		/* There is no server available */
 	    fprintf(out, "450 defer '%s', sender address target '%s' cannot be verified at this time: %s. (Try again later)\r\n",
-		    vaddr->in_addr, vaddr->target,
-		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
+		    vaddr->in_addr, vaddr->target, binderrmsg);
 	    fflush(out);
 	    xfree(vaddr->in_addr);
 	    xfree(vaddr->work_addr);
@@ -1340,12 +1390,11 @@ verify_sender(pp_sender, raw_sender, out)
 	case DB_FAIL:			/* bad DNS request? */
 	case FILE_FAIL:			/* major DNS error! */
 	    fprintf(out, "550 '%s' failure trying to verify address target '%s': %s.\r\n",
-		    vaddr->in_addr, vaddr->target,
-		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
+		    vaddr->in_addr, vaddr->target, binderrmsg);
 	    fflush(out);
 	    write_log(WRITE_LOG_SYS, "remote MAIL FROM: '%s' target '%s': DNS failed: %s; by %s%s%s%s%s%s.",
 		      vaddr->in_addr, vaddr->target,
-		      (binderr->message && *binderr->message) ? binderr->message : "Unknown error",
+		      binderrmsg,
 		      ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 		      sender_host,
 		      sender_host_addr ? " [" : "",
@@ -1358,13 +1407,10 @@ verify_sender(pp_sender, raw_sender, out)
 
 	default:
 	    fprintf(out, "421 '%s' impossible error when trying to verify address target '%s': %s. (Try again later)\r\n",
-		    vaddr->in_addr, vaddr->target,
-		    (binderr->message && *binderr->message) ? binderr->message : "Unknown error");
+		    vaddr->in_addr, vaddr->target, binderrmsg);
 	    fflush(out);
 	    panic(EX_SOFTWARE, "verify_sender(%s): bind_addr(%s) gave impossible result %d: %s.",
-		  vaddr->in_addr, vaddr->target,
-		  (binderr->message && *binderr->message) ? binderr->message : "Unknown error",
-		  mxresult);
+		  vaddr->in_addr, vaddr->target, binderrmsg, mxresult);
 	    /* NOTREACHED */
 	}
     }
@@ -1413,6 +1459,9 @@ verify_addr(in_addr, out, rcpt_p)
     }
     verify_addr_list(addr, &okay, &defer, &fail);
 
+    /* we only submit one address at a time, so it should be on one, and only
+     * one, of the output lists....
+     */
     if (okay) {
 	fprintf(out, "250 '%s' <%s@%s> Recipient Okay.\r\n", in_addr,
 		okay->work_addr, (okay->target) ? okay->target : "(nodomain)");
@@ -1426,7 +1475,8 @@ verify_addr(in_addr, out, rcpt_p)
 		defer->error->info & ERR_MASK, defer->error->message);
 	if (rcpt_p) {
 	    /* XXX this may be very noisy.... */
-	    write_log(WRITE_LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> temporary failure returned to '%s' while verifying recipient: (ERR_%03ld) %s",
+	    write_log(WRITE_LOG_SYS, "remote %s: %s%s%s%s%s%s: '%s' <%s@%s> temporary failure returned to '%s' while verifying recipient: (ERR_%03ld) %s",
+		      rcpt_p ? "RCPT TO" : "VRFY",
 		      ident_sender ? ident_sender : "",
 		      ident_sender ? "@" : "",
 		      sender_host,
@@ -1441,7 +1491,7 @@ verify_addr(in_addr, out, rcpt_p)
 		      defer->error->message);
 	}
 	xfree(addr);
- 	return 1;
+ 	return 0;			/* do not keep this recipient though... */
     } else if (fail) {
 	int error_code;			/* reply code for negative result */
 
@@ -1467,7 +1517,8 @@ verify_addr(in_addr, out, rcpt_p)
 		fail->work_addr, (fail->target) ? fail->target : "(nodomain)",
 		fail->error->info & ERR_MASK, fail->error->message);
 	if (rcpt_p) {
-	    write_log(WRITE_LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched: (ERR_%03ld) %s",
+	    write_log(WRITE_LOG_SYS, "remote %s: %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched: (ERR_%03ld) %s",
+		      rcpt_p ? "RCPT TO" : "VRFY",
 		      ident_sender ? ident_sender : "",
 		      ident_sender ? "@" : "",
 		      sender_host,
@@ -1485,11 +1536,12 @@ verify_addr(in_addr, out, rcpt_p)
  	return 0;
     }
     /* hmmm, it should have been in one of the lists from verify_addr_list()! */
-    fprintf(out, "550 '%s' <%s@%s> does not match anything!\r\n", in_addr,
+    fprintf(out, "550 '%s' <%s@%s> is not deliverable here!\r\n", in_addr,
 	    addr->work_addr, (addr->target) ? addr->target : "(nodomain)");
     if (rcpt_p) {
 	/* XXX do we really need to log these? */
-	write_log(WRITE_LOG_SYS, "remote %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched by anything!",
+	write_log(WRITE_LOG_SYS, "remote %s: %s%s%s%s%s%s: '%s' <%s@%s> recipient for sender '%s' not matched by anything!",
+		  rcpt_p ? "RCPT TO" : "VRFY",
 		  ident_sender ? ident_sender : "", ident_sender ? "@" : "",
 		  sender_host,
 		  sender_host_addr ? "[" : "",
@@ -1512,10 +1564,11 @@ verify_addr(in_addr, out, rcpt_p)
  * (possibly with the text that followed a domain literal form appended).
  */
 static char *
-verify_host(hostnm, saddrp, shp, errorp)
+verify_host(hostnm, saddrp, shp, shpmsg, errorp)
     char *hostnm;
     struct sockaddr_in *saddrp;
-    struct hostent *shp;		/* result of gethostbyaddr() */
+    struct hostent *shp;		/* result of gethostbyaddr() in caller */
+    char const *shpmsg;			/* from gethoostbyaddr() in caller */
     char const **errorp;
 {
     struct hostent *hp;
@@ -1524,6 +1577,12 @@ verify_host(hostnm, saddrp, shp, errorp)
     int has_alphas;
     int i;
     int allowed_to_be_broken = FALSE;
+    int aresult;
+    long bindflgs = BIND_DOMAIN_REQUIRED | BIND_DEFER_NO_CONN;
+    struct rt_info rt_info;
+    struct error *binderr = NULL;
+    char *binderrmsg;
+    static struct bindlib_private bindpriv =  BIND_TEMPLATE_ATTRIBUTES;
 
 # ifndef OBSOLETE_RESOLVER
     extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
@@ -1553,13 +1612,18 @@ verify_host(hostnm, saddrp, shp, errorp)
 	inet = inet_addr(&hostnm[1]);
 # endif
 	*p = ']';
+	DEBUG5(DBG_REMOTE_HI,
+	       "verify_host(%s): inet from helo: [0x%lx] aka %s, actual [0x%lx] aka %s.\n",
+	       hostnm,
+	       (unsigned long) inet, COPY_STRING(inet_ntoa(inet_makeaddr(inet, (unsigned long) 0))), /* HELO */
+	       (unsigned long) saddrp->sin_addr.s_addr, COPY_STRING(inet_ntoa(saddrp->sin_addr))); /* actual */
 	if (inet == (unsigned long)(-1)) {
 	    DEBUG1(DBG_REMOTE_LO, "verify_host(%s): inet_addr() failed: bad host address form.\n", hostnm);
 	    *errorp = "Invalid host address form";
 	    return NULL;
 	}
 	if (inet != saddrp->sin_addr.s_addr) {
-	    DEBUG3(DBG_REMOTE_LO, "verify_host(%s): [0x%lx] != [0x%lx].\n", hostnm, inet, saddrp->sin_addr.s_addr);
+	    DEBUG3(DBG_REMOTE_LO, "verify_host(%s): [0x%lx] != [0x%lx].\n", hostnm, (unsigned long) inet, (unsigned long) saddrp->sin_addr.s_addr);
 	    *errorp = "Host address does not match remote address";
 	    if (smtp_hello_verify && !allowed_to_be_broken) {
 		return NULL;		/* broken DNS or mailer config; or forger */
@@ -1639,7 +1703,17 @@ verify_host(hostnm, saddrp, shp, errorp)
 	    return NULL;
 	}
     }
+#if 0 /* XXX this isn't really necessary with bind_check_if_canonical_host(),
+       * and gives too many hints to spammers
+       */
     if (!has_alphas) {			/* gethostbyname() likes IP #s so don't go there! */
+	/*
+	 * XXX only people using the latest BIND resolver, as I strongly
+	 * recommend, will have inet_aton().  If you get an error about it
+	 * being undefined, and you're cognizant enough to find and read this
+	 * comment, then PLEASE get the latest BIND release and compile & link
+	 * smail against the resolver library it provides!
+	 */
 	if (!inet_aton(hostnm, (struct in_addr *) NULL)) { /* XXX hope all allow NULL! */
 	    *errorp = "not a valid IP literal address, and not surrounted by `[]'";
 	    DEBUG1(DBG_REMOTE_LO, "not a valid literal IP, no []'s: %s.\n", hostnm);
@@ -1653,7 +1727,53 @@ verify_host(hostnm, saddrp, shp, errorp)
 	    return NULL;
 	}
     }
-    /* NOTE: must check shp contents before calling gethost*() again */
+#endif
+
+    /* NOTE:  shp is safe -- this doesn't call any gethostby*() */
+    aresult = bind_check_if_canonical_host(hostnm, saddrp->sin_addr.s_addr, &binderr);
+
+    binderrmsg = (binderr && binderr->message && *binderr->message) ?
+	    binderr->message :
+	    (aresult == DB_SUCCEED) ? "(none)" : "no error messsage given";
+
+    DEBUG4(aresult == DB_SUCCEED ? DBG_REMOTE_HI : DBG_REMOTE_LO,
+	   "verify_host(): bind_check_if_canonical_host(%s, %s): %d, %s.\n",
+	   hostnm, inet_ntoa(saddrp->sin_addr), aresult, binderrmsg);
+
+    switch (aresult) {
+    case DB_SUCCEED:			/* found the host! */
+	break;
+    case FILE_NOMATCH:			/* There is no server available */
+    case DB_NOMATCH:			/* no such domain */
+	*errorp = binderrmsg;
+	if (smtp_hello_verify && !allowed_to_be_broken) {
+	    return NULL;
+	}
+	break;
+    case DB_AGAIN:			/* DNS lookup must be deferred */
+    case FILE_AGAIN:			/* lost contact with server */
+	*errorp = xprintf("DNS failure trying to verify host '%s' (%s)", hostnm, binderrmsg);
+	/* XXX it would be nice to return a temporary error here, but that would mean a re-design */
+	if (smtp_hello_verify && !allowed_to_be_broken) {
+	    return NULL;
+	}
+	break;
+    case DB_FAIL:			/* bad DNS request? */
+    case FILE_FAIL:			/* major DNS error! */
+	*errorp = xprintf("DNS failure trying to verify host '%s' (%s)", hostnm, binderrmsg);
+	/* XXX it might be nice to return a temporary error here, but that would mean a re-design */
+	if (smtp_hello_verify && !allowed_to_be_broken) {
+	    return NULL;
+	}
+    default:				/* impossible! */
+	DEBUG2(DBG_REMOTE_LO, "verify_host(%s): bind_check_if_canonical_host() impossible result %d.", hostnm, aresult);
+	*errorp = xprintf("fatal internal error encountered when verifying host '%s' (%s)", hostnm, binderrmsg);
+	if (smtp_hello_verify && !allowed_to_be_broken) {
+	    return NULL;
+	}
+	break;
+    }
+
     if (shp) {				/* from gethostbyaddr() in caller */
 	int hlen = strlen(hostnm);
 
@@ -1662,7 +1782,10 @@ verify_host(hostnm, saddrp, shp, errorp)
 	    for (i = 0; (p = (shp->h_aliases)[i]); i++) {
 		if (strncmpic(hostnm, p, hlen) == 0) {
 		    found = 1;
-		    /* reset so it'll be == sender_host & not be in Received: */
+		    /*
+		     * reset sender_host_really so it'll be == sender_host & no
+		     * mismatch will be reported in the Received: header
+		     */
 		    if (sender_host_really) {
 			xfree(sender_host_really);
 		    }
@@ -1672,42 +1795,18 @@ verify_host(hostnm, saddrp, shp, errorp)
 	    }
 	    if (!found) {
 		DEBUG2(DBG_REMOTE_LO, "verify_host(%s): gethostbyaddr(%s) PTR does not have matching name.\n", hostnm, inet_ntoa(saddrp->sin_addr));
-		*errorp = "Remote address PTR does not have matching name";
+		*errorp = "Remote address PTR does not have a matching name";
 		if (smtp_hello_verify_ptr && !allowed_to_be_broken) {
 		    return NULL;	/* broken DNS or mailer config; or forger */
 		}
 	    }
 	}
     } else {
-	*errorp = "Remote address PTR lookup failed";
+	*errorp = xprintf("Remote address PTR lookup failed (%s)", shpmsg);
 	if (smtp_hello_verify_ptr && !allowed_to_be_broken) {
 	    return NULL;		/* broken DNS or mailer config; or forger */
 	}
     }
-    if (!(hp = gethostbyname(hostnm))) { /* NOTE: must have checked shp by now.... */
-	*errorp = hstrerror(h_errno);
-	DEBUG2(DBG_REMOTE_LO, "verify_host(%s) gethostbyname() failed: %s.\n", hostnm, hstrerror(h_errno));
-	/* XXX it might be nice to return a temporary error here, but that would mean a re-design */
-	if (smtp_hello_verify && !allowed_to_be_broken) {
-	    return NULL;
-	}
-    } else {
-	found = 0;
-	for (i = 0; hp->h_addr_list[i]; i++) {
-	    if (bcmp(hp->h_addr_list[i], (char *) &saddrp->sin_addr,
-		     sizeof(saddrp->sin_addr)) == 0) {
-		found = 1;
-		break;			/* the name is good, keep it */
-	    }
-	}
-	if (!found) {
-	    DEBUG2(DBG_REMOTE_LO, "verify_host(%s) gethostbyname() doesn't find %s.\n", hostnm, inet_ntoa(saddrp->sin_addr));
-	    *errorp = "Host name does not match remote address";
-	    if (smtp_hello_verify && !allowed_to_be_broken) {
-		return NULL;		/* broken DNS or mailer config; or forger */
-	    }
-	}
-    }
 
     return COPY_STRING(hostnm);
 }
