@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:string.c,v 1.23 1997/12/16 22:12:03 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:string.c,v 1.24 1998/09/27 00:37:06 woods Exp"
  */
 
 /*
@@ -318,7 +318,7 @@ str_printf_va(sp, fmt, ap)
 {
     register int c;			/* current char in fmt */
     register int islong;                /* to handle %ld */
-    long n;                             /* temp */
+    long n;				/* temp */
     char *s;				/* temp */
 
     /*
@@ -346,24 +346,24 @@ str_printf_va(sp, fmt, ap)
 		STR_NEXT(sp, va_arg(ap, int));
 		break;
 	    case 'o':
-		n = islong ? va_arg(ap,long) : (long)va_arg(ap,unsigned);
+		n = islong ? va_arg(ap, unsigned long) : (unsigned long) va_arg(ap, unsigned int);
 		STR_CAT(sp, bltoa(8, n));
 		break;
 	    case 'x':
-		n = islong ? va_arg(ap,long) : (long)va_arg(ap,unsigned);
+		n = islong ? va_arg(ap, unsigned long) : (unsigned long) va_arg(ap, unsigned int);
 		STR_CAT(sp, bltoa(16, n));
 		break;
 	    case 'u':
-		n = islong ? va_arg(ap,long) : (long)va_arg(ap,unsigned);
+		n = islong ? va_arg(ap, unsigned long) : (unsigned long) va_arg(ap, unsigned int);
 		STR_CAT(sp, bltoa(10, n));
 		break;
 	    case 'd':
-		n = islong ? va_arg(ap,long) : (long)va_arg(ap,int);
+		n = islong ? va_arg(ap, long) : (unsigned long) va_arg(ap, int);
 		if (n < 0) {
 		    STR_NEXT(sp, '-');
 		    n = -n;
 		}
-		STR_CAT(sp, bltoa(10, n));
+		STR_CAT(sp, bltoa(10, (unsigned long) n));
 		break;
 	    case 'N':			/* how to insert a nul byte */
 	    case 'z':
@@ -395,7 +395,7 @@ str_printf_va(sp, fmt, ap)
 static char *
 bltoa(base, n)
     register unsigned base;		/* base for conversion */
-    register long     n;                /* number to convert */
+    register unsigned long n;		/* number to convert */
 {
     static char buf[BITS_PER_INT + 1];	/* plenty big */
     register char *p = buf + sizeof(buf); /* start at end and move backward */
