@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:verify.c,v 1.20 1997/12/02 00:23:16 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:verify.c,v 1.21 1998/09/27 00:51:32 woods Exp"
  */
 
 /*
@@ -154,70 +154,82 @@ verify_addr_list(in, okay, defer, fail)
 		*fail = cur;
 #else
 		int mxresult;
+		int local_precedence;
 
-		if ((mxresult = bind_check_if_local_mxs(cur)) >= (DB_SUCCEED + 1)) {
-		    char *dmmytg = NULL;
-		    char *dmmyrem = NULL;
-		    int dmmyflg = 0;
+		switch ((mxresult = bind_check_if_local_mxs(cur, &local_precedence))) {
+		case DB_SUCCEED:
+		    DEBUG1(DBG_ADDR_MID, "bind_check_if_local_mxs() found MXs for %s.", cur->in_addr);
+		    if (local_precedence >= 0) {
+			char *dmmytg = NULL;
+			char *dmmyrem = NULL;
+			int dmmyflg = 0;
 
-		    /* it's OK -- we MX for them, unless... */
-		    if (parse_address(cur->remainder, &dmmytg, &dmmyrem, &dmmyflg) != LOCAL) {
-			illegal_relay_error(cur, sender_host_addr);
-			cur->succ = *fail;
-			*fail = cur;
+			/* it's OK -- we MX for them, unless... */
+			if (parse_address(cur->remainder, &dmmytg, &dmmyrem, &dmmyflg) != LOCAL) {
+			    illegal_relay_error(cur, sender_host_addr);
+			    cur->succ = *fail;
+			    *fail = cur;
+			} else {
+			    cur->succ = remote;
+			    remote = cur;
+			}
 		    } else {
-			cur->succ = remote;
-			remote = cur;
-		    }
-		} else {
-		    switch (mxresult) {
-		    case DB_SUCCEED:
 			illegal_relay_error(cur, sender_host_addr);
 			cur->succ = *fail;
 			*fail = cur;
-			break;
-		    case DB_AGAIN:	/* DNS lookup must be deferred */
-		    case FILE_AGAIN:	/* lost contact with server, etc. */
-		    case FILE_NOMATCH:	/* could not connect to server */
-			if (!cur->error) {
-			    char *error_text;
-
-			    error_text = xprintf("defer checking for MXs for %s",
-						 cur->in_addr);
-			    cur->error = note_error(ERR_DONTLOG | ERR_163, error_text);
-			}
-			cur->succ = *defer;
-			*defer = cur;
-			break;
-		    case DB_NOMATCH:	/* no such domain */
-		    case DB_FAIL:	/* bad DNS request? */
-			if (!cur->error) {
-			    char *error_text;
-
-			    error_text = xprintf("%s is not a valid domain",
-						 cur->in_addr);
-			    cur->error = note_error(ERR_NPOWNER | ERR_164, error_text);
-			}	
-			cur->succ = *fail;
-			*fail = cur;
-			break;
-		    case FILE_FAIL:		/* major DNS error! */
-			if (!cur->error) {
-			    char *error_text;
-
-			    error_text = xprintf("error checking for MXs for %s",
-						 cur->in_addr);
-			    cur->error = note_error(ERR_NPOWNER | ERR_165, error_text);
-			}
-			cur->succ = *fail;
-			*fail = cur;
-			break;
-		    default:
-			panic(EX_SOFTWARE,
-			      "bind_check_if_local_mxs() impossible result %d.",
-			      mxresult);
-			/* NOTREACHED */
 		    }
+		    break;
+		case DB_AGAIN:		/* DNS lookup must be deferred */
+		case FILE_AGAIN:	/* lost contact with server, etc. */
+		case FILE_NOMATCH:	/* could not connect to server */
+		    DEBUG1(DBG_ADDR_LO, "bind_check_if_local_mxs() DNS comms problem when checking for MXs for %s.", cur->in_addr);
+		    if (!cur->error) {
+			char *error_text;
+
+			error_text = xprintf("defer checking for MXs for %s",
+					     cur->in_addr);
+			cur->error = note_error(ERR_DONTLOG | ERR_163, error_text);
+		    }
+		    cur->succ = *defer;
+		    *defer = cur;
+		    break;
+		case DB_NOMATCH:	/* no such domain */
+		case DB_FAIL:		/* bad DNS request? */
+		    DEBUG1(DBG_ADDR_LO, "bind_check_if_local_mxs() NXDOMAIN or bad request when checking for MXs for %s.", cur->in_addr);
+		    if (!cur->error) {
+			char *error_text;
+
+			error_text = xprintf("%s is not a valid domain",
+					     cur->in_addr);
+			cur->error = note_error(ERR_NPOWNER | ERR_164, error_text);
+		    }	
+		    cur->succ = *fail;
+		    *fail = cur;
+		    break;
+		case FILE_FAIL:		/* major DNS error! */
+		    DEBUG1(DBG_ADDR_LO, "bind_check_if_local_mxs() major DNS error when checking for MXs for %s.", cur->in_addr);
+		    if (!cur->error) {
+			char *error_text;
+
+			error_text = xprintf("error checking for MXs for %s",
+					     cur->in_addr);
+			cur->error = note_error(ERR_NPOWNER | ERR_165, error_text);
+		    }
+		    cur->succ = *fail;
+		    *fail = cur;
+		    break;
+		default:
+		    DEBUG2(DBG_ADDR_LO, "bind_check_if_local_mxs() impossible result %d when checking for MXs for %s.", mxresult, cur->in_addr);
+		    if (!cur->error) {
+			char *error_text;
+
+			error_text = xprintf("fatal internal error %d checking for MXs for %s.",
+					     mxresult, cur->in_addr);
+			cur->error = note_error(ERR_NPOWNER | ERR_165, error_text);
+		    }
+		    cur->succ = *fail;
+		    *fail = cur;
+		    break;
 		}
 #endif /* HAVE_BIND */
 	    } else {
