@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/src:RELEASE-3_2_0_102:mkaliases.sh,v 1.12 1998/04/11 23:28:07 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_103:mkaliases.sh,v 1.14 1998/10/01 05:50:12 woods Exp"
 #
 #    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
 #    Copyright (C) 1992  Ronald S. Karr
@@ -33,6 +33,7 @@ ALIASES_TYPE='X_ALIASES_TYPE_X'
 
 prog=$0
 GETOPT="X_UTIL_BIN_DIR_X/getopt"
+DEBUG=0
 
 USAGE="Usage: $prog [-v N] [alias_file [alias_type]"
 
@@ -40,7 +41,7 @@ set -- `$GETOPT v: $*`
 
 if [ $? -ne 0 ]; then
 	echo $USAGE 1>&2
-	exit 2
+	exit 64			# EX_USAGE
 fi
 
 for i in $*; do
@@ -62,12 +63,20 @@ fi
 
 if [ ! -w "$ALIASES_FILE" ] ; then
 	echo "$prog: '$ALIASES_FILE' does not appear to be a regular writable file." 1>&2
-	exit 1
+	exit 77			#  EX_NOPERM
 fi
 
 if [ ! -w "`dirname $ALIASES_FILE`" ] ; then
 	echo "$prog: directory '`dirname $ALIASES_FILE`' does not appear to be writable." 1>&2
-	exit 1
+	exit 77			#  EX_NOPERM
+fi
+
+if [ "$DEBUG" -gt 5 ] ; then
+	echo "$prog: running as: `id`"
+fi
+
+if [ "$DEBUG" -gt 1 ] ; then
+	echo "$prog: processing $ALIASES_FILE as type $ALIASES_TYPE"
 fi
 
 # NOTE: list of possible types defined in lookup.c:protos[]
@@ -75,9 +84,19 @@ fi
 case "$ALIASES_TYPE" in
 aliasyp | yp | nialias | nisplus )
 	mkline "$ALIASES_FILE" | mkdbm -y -v -o "$ALIASES_FILE"
+	rc=$?
+	if [ $rc -ne 0 ] ; then
+		echo "$prog: failed to create YP/NIS DBM file for '$ALIASES_FILE'." 1>&2
+		exit $rc
+	fi
 	;;
 dbm )
 	mkline "$ALIASES_FILE" | mkdbm -f -v -o "$ALIASES_FILE"
+	rc=$?
+	if [ $rc -ne 0 ] ; then
+		echo "$prog: failed to create DBM file for '$ALIASES_FILE'." 1>&2
+		exit $rc
+	fi
 	;;
 bsearch )
 	ALIASES_DIR=`dirname "$ALIASES_FILE"`
@@ -89,21 +108,35 @@ bsearch )
 		TEXT_FILE="`basename "$ALIASES_FILE"`".txt
 		;;
 	esac
+	if [ ! -r "$ALIASES_DIR/$TEXT_FILE" ] ; then
+		echo "$prog: $ALIASES_DIR/$TEXT_FILE: unable to read file." 1>&2
+		exit 77		#  EX_NOPERM
+	fi
 	mkline "$ALIASES_DIR/$TEXT_FILE" | wc -lc | awk '{ print $1 " entries, " $2 " bytes total" }'
 	rc=$?
 	if [ $rc -ne 0 ] ; then
-		echo "$0: not rebuilding '$ALIASES_FILE'." 1>&2
+		echo "$prog: not rebuilding '$ALIASES_FILE'." 1>&2
 		exit $rc
 	fi
 	mkline "$ALIASES_DIR/$TEXT_FILE" | mksort -f > "$ALIASES_DIR/.$TEXT_FILE"
+	rc=$?
+	if [ $rc -ne 0 ] ; then
+		echo "$prog: failed to rebuild '$ALIASES_FILE' from '$ALIASES_DIR/$TEXT_FILE'." 1>&2
+		exit $rc
+	fi
 	mv -f "$ALIASES_DIR/.$TEXT_FILE" "$ALIASES_FILE"
 	;;
 lsearch )
 	mkline "$ALIASES_FILE" | wc -lc | awk '{ print $1 " entries, " $2 " bytes total" }'
+	rc=$?
+	if [ $rc -ne 0 ] ; then
+		echo "$prog: failed to parse '$ALIASES_FILE'." 1>&2
+		exit $rc
+	fi
 	;;
 * )
-	echo "$0: unknown file type, $ALIASES_TYPE, for $ALIASES_FILE" 1>&2
-	exit 1
+	echo "$prog: $ALIASES_FILE: unknown file type: $ALIASES_TYPE" 1>&2
+	exit 78			# EX_CONFIG
 	;;
 esac
 
