@@ -1,4 +1,6 @@
-/* @(#) mkline.c,v 1.5 1997/12/16 22:07:34 woods Exp */
+/*
+#ident	"@(#)smail/util:RELEASE-3_2_0_103:mkline.c,v 1.6 1998/10/07 05:20:40 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -152,7 +154,16 @@ mkaliasline(f)
 	error = tokenize(sp, &tokens, TRUE, TRUE);
 
 	if (error) {
-	    (void) fprintf(stderr, "%s: tokenize error: %s\n", program, error);
+	    if (ignore_keys_p) {	/* wasn't split above... */
+		sp = s;
+		while (!isspace(*sp) && *sp != ':') {
+		    sp++;
+		}
+
+		/* split the line in two */
+		*sp++ = '\0';
+	    }
+	    (void) fprintf(stderr, "%s: tokenize error in '%s': %s\n", program, s, error);
 	    return -1;
 	}
 
